﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Docx
{
    using DocumentFormat.OpenXml;
    using DocumentFormat.OpenXml.Packaging;
    using DocumentFormat.OpenXml.Wordprocessing;
    using System.Globalization;
    using static FormatowaniaDocx;

    public static class FormatowaniaDocx
    {
        private const int rozmiarCzcionkiNormalny = 21;
        private const int rozmiarCzcionkiTytułu = 48;
        private const string kształtCzcionki = "Arial";

        public static Color KolorWyróżniony
        {
            get
            {
                return new Color() { Val = "000055" };
            }
        }

        public static Color KolorNormalny
        {
            get
            {
                return new Color() { Val = "black" };
            }
        }

        public static RunFonts CzcionkaNormalna
        {
            get
            {
                return new RunFonts()
                {
                    Ascii = kształtCzcionki,
                    HighAnsi = kształtCzcionki
                };
            }
        }

        public static RunProperties StylUstępuNormalny
        {
            get 
            {
                return new RunProperties()
                {
                    RunFonts = CzcionkaNormalna,
                    FontSize = new FontSize()
                    {
                        Val = new StringValue(rozmiarCzcionkiNormalny.ToString())
                    },
                    Color = KolorNormalny
                };
            }
        }

        public static RunProperties StylUstępuTytułu
        {
            get
            {
                return new RunProperties()
                {
                    Bold = new Bold()
                    {
                        Val = OnOffValue.FromBoolean(true)
                    },
                    RunFonts = CzcionkaNormalna,
                    FontSize = new FontSize()
                    {
                        Val = new StringValue(rozmiarCzcionkiTytułu.ToString())
                    },
                    Color = KolorWyróżniony
                };
            }
        }

        public static ParagraphProperties UstawieniaAkapituZwiększonaInterlinia
        {
            get
            {
                ParagraphProperties paragraphProperties = new ParagraphProperties();
                paragraphProperties.Append(
                    new SpacingBetweenLines() { Line = "360" });
                return paragraphProperties;
            }
        }
    }

    public static class PomocnikDocx
    {
        private static Paragraph twórzAkapit(string[] linie, int rozmiarCzcionki, Color kolorCzcionki, bool bezInterlinii = false)
        {
            Paragraph akapit = new Paragraph();
            if(!bezInterlinii) akapit.Append(UstawieniaAkapituZwiększonaInterlinia);
            Run ustęp = akapit.AppendChild(new Run());
            RunProperties stylAkapitu = StylUstępuNormalny;
            stylAkapitu.FontSize.Val = new StringValue(rozmiarCzcionki.ToString());
            stylAkapitu.Color = kolorCzcionki;
            ustęp.Append(stylAkapitu);
            foreach(string linia in linie)
            {
                ustęp.AppendChild(new Text(linia));
                ustęp.AppendChild(new Break());
            }
            return akapit;
        }

        private static Paragraph twórzAkapitTytułu(string tytuł)
        {
            Paragraph akapit = new Paragraph();
            Run ustępTytułu = akapit.AppendChild(new Run());
            ustępTytułu.Append(StylUstępuTytułu);
            ustępTytułu.AppendChild(new Text(tytuł));
            return akapit;
        }

        private static Stream pobierzRysunekZPliku(string ścieżkaDoPliku)
        {
            Stream stream = new FileStream(ścieżkaDoPliku, FileMode.Open);
            return stream;
        }

        private static Drawing twórzRysunekWDokumencie(WordprocessingDocument dokumentWorda, Stream strumień)
        {
            MainDocumentPart częśćGłówna = dokumentWorda.MainDocumentPart;
            ImagePart częśćRysunku = częśćGłówna.AddImagePart(ImagePartType.Png);
            częśćRysunku.FeedData(strumień);
            return RysunekDocx.CreateDrawingElement(dokumentWorda, częśćGłówna.GetIdOfPart(częśćRysunku), 3, 1.5f);            
        }

        private static Paragraph twórzAkapit(Drawing rysunek)
        {
            return new Paragraph(new Run(rysunek));
        }

        private static void wyślijDokumentDoStrumienia(Raport raport, Stream stream, IFormatProvider formatProvider)
        {
            using(WordprocessingDocument dokumentWorda = WordprocessingDocument.Create(
                stream, WordprocessingDocumentType.Document, true))
            {
                MainDocumentPart głównaCzęśćDokumentu = dokumentWorda.AddMainDocumentPart();
                głównaCzęśćDokumentu.Document = new Document();

                Body ciałoDokumentu = new Body();
                głównaCzęśćDokumentu.Document.AppendChild(ciałoDokumentu);

                Stream strumieńRysunku = pobierzRysunekZPliku("UMK.png");
                Drawing rysunek = twórzRysunekWDokumencie(dokumentWorda, strumieńRysunku);
                Paragraph akapitObrazu = twórzAkapit(rysunek);
                ciałoDokumentu.AppendChild(akapitObrazu);

                string[] linieNagłówka =
                {
                    "e-mail: jacek@fizyka.umk.pl",
                    "www.umk.pl",
                    "tel.: +48 56 6111 3310"
                };
                Paragraph akapitNagłówka = twórzAkapit(linieNagłówka, 18, KolorWyróżniony, true);
                ciałoDokumentu.AppendChild(akapitNagłówka);

                ciałoDokumentu.AppendChild(twórzAkapitTytułu("Raport"));

                //akapit
                Paragraph akapit = twórzAkapit(raport.Linie, 25, KolorNormalny);
                ciałoDokumentu.AppendChild(akapit);

                dokumentWorda.Close();
            }
            stream.Flush();
        }

        public static void EksportujDoPlikuDocx(this Raport raport, string ścieżkaDoPliku, IFormatProvider format = null)
        {
            if (format == null) format = new CultureInfo("pl-PL");
            if (File.Exists(ścieżkaDoPliku)) File.Delete(ścieżkaDoPliku);
            using(FileStream strumieńPliku = new FileStream(ścieżkaDoPliku, FileMode.CreateNew))
            {
                wyślijDokumentDoStrumienia(raport, strumieńPliku, format);
                strumieńPliku.Close();
            }
        }
    }
}
